/*******************************************************************************************************************************************************************************************************************************************************************************************************************************************************************************************************************************************************************************************************************
 *
 * @author Karolina Krolik <<ahref='mailto:k.krolik@samsung.com'>k.krolik@samsung.com</a>>, Tadeusz Wlodarkiewicz <<ahref='mailto:t.wlodarkiew@samsung.com'>t.wlodarkiew@samsung.com</a>>
 *
 *
 * ********************************************************************************************************
 *
 * Copyright (c) 2012 Samsung Electronics All Rights Reserved.
 *
 ******************************************************************************************************************************************************************************************************************************************************************************************************************************************************************************************************************************************************************************************************************/

/**
 * Helps game module with calculating some distance and score
 */
gameUtils = function($) {

    return {

        /**
         * Calculate distance between two points
         *
         * @param point1
         *            the first point (b2Vec2 object)
         * @param point2
         *            the second point (b2Vec2 object)
         * @returns distance between two points
         */
        distance : function(point1, point2) {
            x = (point2.x - point1.x);
            y = (point2.y - point1.y);

            return Math.sqrt(x * x + y * y);
        },

        /**
         * Compare b2Vec2 velocity vector with given treshold value (velocity
         * below than given value)
         *
         * @param vec
         *            vector to compare
         * @param treshold
         *            treshold to compare with
         * @returns true if both vec (velocity) coordinates are below the
         *          treshold, false otherwise
         */
        compareVelocity : function(vec, treshold) {
            if (vec.x > treshold) {
                return false;
            }
            if (vec.y > treshold) {
                return false;
            }
            return true;
        },

        /**
         * Calculate game results, sum of boules' distances from the jack for
         * each player
         *
         * @param spheres
         *            Array of spheres:inedxes 0-1 player one, 2-3 player two, 4
         *            main sphere (b2Body objects)
         * @returns Array with results: result[0] - winner number (1 or 2),
         *          reslut[1] - player one result, result[2] - player two result
         */
        checkReults : function(spheres) {

            var spheresPos = [];
            for ( var i = 0; i < spheres.length; i++) {
                var pos = spheres[i].GetPosition();
                var newX = pos.x;
                var newY = pos.y;
                spheresPos.push({
                    x : newX,
                    y : newY
                });
            }

            var spheresP1 = spheresPos.slice(0, 2);
            var spheresP2 = spheresPos.slice(2, 4);
            var sphereMain = spheresPos[4];

            var distanceP1 = gameUtils.distance(spheresP1[0], sphereMain)
                    + gameUtils.distance(spheresP1[1], sphereMain);
            var distanceP2 = gameUtils.distance(spheresP2[0], sphereMain)
                    + gameUtils.distance(spheresP2[1], sphereMain);

            var result = [];
            if (distanceP1 < distanceP2) {
                result.push(1);
            } else {
                result.push(2);
            }
            result.push(distanceP1.toFixed(3));
            result.push(distanceP2.toFixed(3));

            return result;
        }

    };
}($);
