/*******************************************************************************************************************************************************************************************************************************************************************************************************************************************************************************************************************************************************************************************************************
 *
 * @author Tadeusz Wlodarkiewicz <<ahref='mailto:t.wlodarkiew@samsung.com'>t.wlodarkiew@samsung.com</a>>, Karolina Krolik <<ahref='mailto:k.krolik@samsung.com'>k.krolik@samsung.com</a>>
 *
 *
 * ********************************************************************************************************
 *
 * Copyright (c) 2012 Samsung Electronics All Rights Reserved.
 *
 ******************************************************************************************************************************************************************************************************************************************************************************************************************************************************************************************************************************************************************************************************************/

/**
 * Creates ground in webgl and loads it into webgl buffers. Can set buffers as
 * active and draw ground when needed.
 */
var ground = function($) {
    // reference to webgl
    var gl;
    // reference to shader
    var shaderProgram;

    // Ground position vertices
    var groundVertexPositionBuffer;
    // Ground texture coordinates
    var groundVertexTextureCoordBuffer;
    // Ground indices
    var groundVertexIndexBuffer;
    // Reference to ground loaded texture
    var texture;

    /**
     * Creates vertices, texture coordinates and indices for ground and loads
     * them to the webgl buffers
     */
    var createGround = function() {
        groundVertexPositionBuffer = gl.createBuffer();
        gl.bindBuffer(gl.ARRAY_BUFFER, groundVertexPositionBuffer);
        vertices = [ -1.0, -1.0, 0.0, 1.0, -1.0, 0.0, 1.0, 1.0, 0.0, -1.0, 1.0, 0.0,

        -1.0, -1.0, -0.1, 1.0, -1.0, -0.1, 1.0, 1.0, -0.1, -1.0, 1.0, -0.1

        ];
        gl.bufferData(gl.ARRAY_BUFFER, new Float32Array(vertices), gl.STATIC_DRAW);
        groundVertexPositionBuffer.itemSize = 3;
        groundVertexPositionBuffer.numItems = 8;

        groundVertexTextureCoordBuffer = gl.createBuffer();
        gl.bindBuffer(gl.ARRAY_BUFFER, groundVertexTextureCoordBuffer);
        var textureCoords = [ 0.0, 0.0, 1.0, 0.0, 1.0, 1.0, 0.0, 1.0,

        0.0, 0.0, 1.0, 0.0, 1.0, 1.0, 0.0, 1.0 ];
        gl.bufferData(gl.ARRAY_BUFFER, new Float32Array(textureCoords), gl.STATIC_DRAW);
        groundVertexTextureCoordBuffer.itemSize = 2;
        groundVertexTextureCoordBuffer.numItems = 4;

        groundVertexIndexBuffer = gl.createBuffer();
        gl.bindBuffer(gl.ELEMENT_ARRAY_BUFFER, groundVertexIndexBuffer);
        var cubeVertexIndices = [ 0, 1, 2, 0, 2, 3, 0, 4, 3, 4, 7, 3, 0, 4, 1, 1, 5, 4, 1, 5, 6, 1,
                6, 2, 3, 7, 2, 2, 6, 7, 7, 4, 6, 6, 4, 5 ];
        gl.bufferData(gl.ELEMENT_ARRAY_BUFFER, new Uint16Array(cubeVertexIndices), gl.STATIC_DRAW);
        groundVertexIndexBuffer.itemSize = 1;
        groundVertexIndexBuffer.numItems = 36;
    };

    /**
     * Loads texture to webgl buffer after loading texture image
     *
     * @param texture
     *            file that is successfully loaded
     */
    var handleLoadedTexture = function(texture) {
        gl.bindTexture(gl.TEXTURE_2D, texture);
        gl.pixelStorei(gl.UNPACK_FLIP_Y_WEBGL, true);
        gl.texImage2D(gl.TEXTURE_2D, 0, gl.RGBA, gl.RGBA, gl.UNSIGNED_BYTE, texture.image);
        gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_MAG_FILTER, gl.NEAREST);
        gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_MIN_FILTER, gl.NEAREST);
        gl.bindTexture(gl.TEXTURE_2D, null);
    };

    /**
     * Initiates ground texture
     */
    var initTexture = function() {
        texture = gl.createTexture();
        texture.image = new Image();
        texture.image.onload = function() {
            handleLoadedTexture(texture);
        };

        texture.image.src = "images/Grass.jpg";
    };

    return {
        /**
         * Initiates ground module, start texture loading and creating ground
         * faces
         *
         * @param glParam
         *            reference to webgl
         * @param shaderProgramParam
         *            reference to shader
         */
        initGround : function(glParam, shaderProgramParam) {
            gl = glParam;
            shaderProgram = shaderProgramParam;
            initTexture();
            createGround();
        },

        /**
         * Sets ground buffers as active buffers in webgl, so that ground can be
         * drawn
         */
        loadGround : function() {
            gl.bindBuffer(gl.ARRAY_BUFFER, groundVertexPositionBuffer);
            gl.vertexAttribPointer(shaderProgram.vertexPositionAttribute,
                    groundVertexPositionBuffer.itemSize, gl.FLOAT, false, 0, 0);

            gl.bindBuffer(gl.ARRAY_BUFFER, groundVertexTextureCoordBuffer);
            gl.vertexAttribPointer(shaderProgram.textureCoordAttribute,
                    groundVertexTextureCoordBuffer.itemSize, gl.FLOAT, false, 0, 0);

            gl.activeTexture(gl.TEXTURE0);
            gl.bindTexture(gl.TEXTURE_2D, texture);
            gl.uniform1i(shaderProgram.samplerUniform, 0);

            gl.bindBuffer(gl.ELEMENT_ARRAY_BUFFER, groundVertexIndexBuffer);
        },

        /**
         * Draws ground in webgl
         */
        drawGround : function() {
            gl.drawElements(gl.TRIANGLES, groundVertexIndexBuffer.numItems, gl.UNSIGNED_SHORT, 0);
        }
    };
}($);