/*******************************************************************************
 * Tizen application life cycle handler
 *
 *
 * @author Tomasz Scislo <<ahref='mailto:t.scislo@samsung.com'>t.scislo@samsung.com</a>>
 *
 *
 * **************************************************************************************
 *
 * Copyright (c) 2012 Samsung Electronics All Rights Reserved.
 *
 ******************************************************************************/
var tizenVisibility = function() {

    var states = {
        HIDDEN : "hidden",
        VISIBLE : "visible",
        PRERENDER : "prerender",
        UNLOADED : "unloaded"
    };

    var visibilityChanged = function() {
        switch (document.webkitVisibilityState) {
        case states.PRERENDER:
        case states.UNLOADED:
            break;
        case states.HIDDEN:
            onApplicationHidden();
            break;
        case states.VISIBLE:
            onApplicationVisible();
            break
        default:
            break;
        }
    };

    onApplicationVisible = function() {
    };
    onApplicationHidden = function() {
    };

    document.addEventListener("webkitvisibilitychange", visibilityChanged);

    return {

        /**
         * Method to register onApplicationVisible event
         *
         * @param callback
         *            {Function} - method to be called each time application
         *            goes to foreground
         */
        onApplicationVisible : function(callback) {
            onApplicationVisible = callback;
        },

        /**
         * Method to register onApplicationHidden event
         *
         * @param callback
         *            {Function} - method to be called each time application
         *            goes to background
         */
        onApplicationHidden : function(callback) {
            onApplicationHidden = callback;
        }
    };
};

if (typeof tizen !== "undefined" && typeof tizen.application !== "undefined") {
    tizen.application.visibility = tizenVisibility();
} else {
    tizenVisibility = tizenVisibility();
}

/**
 * EXAMPLE 1
 *
 */
/*
 * In case tizen.application is NOT turn on in config.xml
 *
 * tizenVisibility.onApplicationVisible(function() { console.log("app visible");
 * });
 *
 * tizenVisibility.onApplicationHidden(function() { console.log("app hidden");
 * });
 */

/**
 * EXAMPLE 2
 *
 */
/*
 * In case tizen.application is turn on in config.xml
 *
 * tizen.application.visibility.onApplicationVisible(function() {
 * console.log("app visible"); });
 *
 * tizen.application.visibility.onApplicationHidden(function() {
 * console.log("app hidden"); });
 */
