/*******************************************************************************************************************************************************************************************************************************************************************************************************************************************************************************************************************************************************************************************************************
 *
 * @author Tadeusz Wlodarkiewicz <<ahref='mailto:t.wlodarkiew@samsung.com'>t.wlodarkiew@samsung.com</a>>, Karolina Krolik <<ahref='mailto:k.krolik@samsung.com'>k.krolik@samsung.com</a>>
 *
 *
 * ********************************************************************************************************
 *
 * Copyright (c) 2012 Samsung Electronics All Rights Reserved.
 *
 ******************************************************************************************************************************************************************************************************************************************************************************************************************************************************************************************************************************************************************************************************************/

/**
 * The following code was created with help and using source code from Learning WebGL Tutorial Website.
 * Learning WebGL author: Giles Thomas
 * Learning WebGL address: http://learningwebgl.com
 *
 */

/**
 * Handles creating, loading and drawing spheres
 */
var spheres = function($) {
    // reference to webgl
    var gl;
    // reference to shader
    var shaderProgram;

    // types of spheres, filled in the create sphere method
    var sphereTypes = [];

    // influences sphere quality
    var latitudeBands = 15;
    var longitudeBands = 15;

    // image file paths to the textures
    var textureSources = [ "images/grey_marble.png", "images/blue_stripes.png",
            "images/blue_stripesI.png", "images/red_stripes.png", "images/red_stripesI.png" ];

    // array of the loaded textures
    var textures = [];

    /**
     * Loads textures defined in the textureSources variable
     */
    var initTextures = function() {

        for ( var i = 0; i < textureSources.length; i++) {
            textures[i] = gl.createTexture();
            textures[i].image = new Image();
            textures[i].image.src = textureSources[i];

            (function(iRef) {

                textures[iRef].image.onload = function() {
                    textures[iRef].image;
                    gl.bindTexture(gl.TEXTURE_2D, textures[iRef]);
                    gl.pixelStorei(gl.UNPACK_FLIP_Y_WEBGL, true);
                    gl.texImage2D(gl.TEXTURE_2D, 0, gl.RGBA, gl.RGBA, gl.UNSIGNED_BYTE,
                            textures[iRef].image);
                    gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_MAG_FILTER, gl.NEAREST);
                    gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_MIN_FILTER, gl.NEAREST);
                    gl.bindTexture(gl.TEXTURE_2D, null);

                };

            }(i));
        }
    };

    return {
        initSpheres : function(glParam, shaderProgramParam) {
            gl = glParam;
            shaderProgram = shaderProgramParam;
            initTextures();
        },

        /**
         * Creates a sphere with defined type and radius and loads it to the
         * buffer
         *
         * @param radius
         *            {Number} radius of the sphere
         * @param type
         *            {Number} type of the sphere
         */
        createSphere : function(radius, type) {
            var vertexPositionData = [];
            var textureCoordData = [];
            var latNumber = 0;
            var longNumber = 0;
            for (latNumber = 0; latNumber <= latitudeBands; latNumber++) {
                var theta = latNumber * Math.PI / latitudeBands;
                var sinTheta = Math.sin(theta);
                var cosTheta = Math.cos(theta);

                for (longNumber = 0; longNumber <= longitudeBands; longNumber++) {
                    var phi = longNumber * 2 * Math.PI / longitudeBands;
                    var sinPhi = Math.sin(phi);
                    var cosPhi = Math.cos(phi);

                    var x = cosPhi * sinTheta;
                    var y = cosTheta;
                    var z = sinPhi * sinTheta;
                    var u = 1 - (longNumber / longitudeBands);
                    var v = latNumber / latitudeBands;

                    textureCoordData.push(u);
                    textureCoordData.push(v);
                    vertexPositionData.push(radius * x);
                    vertexPositionData.push(radius * y);
                    vertexPositionData.push(radius * z);
                }
            }

            var indexData = [];
            for (latNumber = 0; latNumber < latitudeBands; latNumber++) {
                for (longNumber = 0; longNumber < longitudeBands; longNumber++) {
                    var first = (latNumber * (longitudeBands + 1)) + longNumber;
                    var second = first + longitudeBands + 1;
                    indexData.push(first);
                    indexData.push(second);
                    indexData.push(first + 1);

                    indexData.push(second);
                    indexData.push(second + 1);
                    indexData.push(first + 1);
                }
            }

            var vertexTextureCoordBuffer = gl.createBuffer();
            var vertexPositionBuffer = gl.createBuffer();
            var vertexIndexBuffer = gl.createBuffer();
            sphereTypes.push({
                vertexPositionBuffer : vertexPositionBuffer,
                vertexTextureCoordBuffer : vertexTextureCoordBuffer,
                vertexIndexBuffer : vertexIndexBuffer
            });

            gl.bindBuffer(gl.ARRAY_BUFFER, sphereTypes[type].vertexTextureCoordBuffer);
            gl.bufferData(gl.ARRAY_BUFFER, new Float32Array(textureCoordData), gl.STATIC_DRAW);
            sphereTypes[type].vertexTextureCoordBuffer.itemSize = 2;
            sphereTypes[type].vertexTextureCoordBuffer.numItems = textureCoordData.length / 2;

            gl.bindBuffer(gl.ARRAY_BUFFER, sphereTypes[type].vertexPositionBuffer);
            gl.bufferData(gl.ARRAY_BUFFER, new Float32Array(vertexPositionData), gl.STATIC_DRAW);
            sphereTypes[type].vertexPositionBuffer.itemSize = 3;
            sphereTypes[type].vertexPositionBuffer.numItems = vertexPositionData.length / 3;

            gl.bindBuffer(gl.ELEMENT_ARRAY_BUFFER, sphereTypes[type].vertexIndexBuffer);
            gl.bufferData(gl.ELEMENT_ARRAY_BUFFER, new Uint16Array(indexData), gl.STREAM_DRAW);
            sphereTypes[type].vertexIndexBuffer.itemSize = 3;
            sphereTypes[type].vertexIndexBuffer.numItems = indexData.length;

        },

        /**
         * Loading sphere to the current buffer. After that the sphere can be
         * drown using drawSphere method.
         *
         * @param type
         *            {Number} type of the sphere to be drawn
         */
        loadSphere : function(type) {
            gl.bindBuffer(gl.ARRAY_BUFFER, sphereTypes[type].vertexPositionBuffer);
            gl.vertexAttribPointer(shaderProgram.vertexPositionAttribute,
                    sphereTypes[type].vertexPositionBuffer.itemSize, gl.FLOAT, false, 0, 0);

            gl.bindBuffer(gl.ARRAY_BUFFER, sphereTypes[type].vertexTextureCoordBuffer);
            gl.vertexAttribPointer(shaderProgram.textureCoordAttribute,
                    sphereTypes[type].vertexTextureCoordBuffer.itemSize, gl.FLOAT, false, 0, 0);

            gl.activeTexture(gl.TEXTURE0);
            gl.bindTexture(gl.TEXTURE_2D, textures[type]);
            gl.uniform1i(shaderProgram.samplerUniform, 0);

            gl.bindBuffer(gl.ELEMENT_ARRAY_BUFFER, sphereTypes[type].vertexIndexBuffer);
        },

        /**
         * Draws the specified sphere on the canvas
         *
         * @param type
         *            {Number} type of sphere to be drawn
         */
        drawSphere : function(type) {
            gl.drawElements(gl.TRIANGLES, sphereTypes[type].vertexIndexBuffer.numItems,
                    gl.UNSIGNED_SHORT, 0);
        }
    };
}($);
